(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.normalizeText = {}));
}(this, (function (exports) { 'use strict';

    /**
     * Capitalize first character of received text.
     * @example
     * capitalizeFirstLetter('karl Marx');
     * //=> "Karl Marx"
     * @param {string} text - A `string` value.
     * @returns {string}
     */
    const capitalizeFirstLetter = (text) => text.charAt(0).toLocaleUpperCase() + text.substring(1);

    /**
     * If `String.prototype.normalize` is supported it normalizes diacritics by
     * replacing them with "clean" character from received text.
     * @example
     * normalizeDiacritics('Olá, você aí');
     * //=> 'Ola, voce ai'
     * @param {string} text - A `string` value.
     * @returns {string}
     */
    const normalizeDiacritics = (text) => !!String.prototype.normalize
        ? text.normalize('NFKD').replace(/[\u0080-\uF8FF]/g, '')
        : text;

    /**
     * Normalize all white-space characters and remove trailing ones received text.
     * @example
     * normalizeWhiteSpaces(' What exactly is it?   ');
     * //=> "What exactly is it?"
     *
     * normalizeWhiteSpaces('Hi,   how is \r\n everything  \t?');
     * //=> 'Hi, how is everything ?'
     * ```
     * @param {string} text - A `string` value.
     * @returns {string}
     */
    const normalizeWhiteSpaces = (text) => text.replace(/\s+/g, ' ').trim();

    /**
     * Split text into words.
     * @param {string} text - A `string` value.
     * @returns {string[]}
     */
    const splitWords = (text) => normalizeWhiteSpaces(text.toLocaleLowerCase()).split(' ');
    /**
     * Capitalize word if it isn't an exception.
     * @param {string[]} exceptions - A list of exceptions in lower-case.
     * @returns {(word: string) => string}
     */
    const capitalizeWordExcept = (exceptions) => (word) => exceptions.indexOf(word) > -1 ? word : capitalizeFirstLetter(word);
    /**
     * Normalize received name by normalizing it's white-spaces and capitalizing
     * first letter of every word but exceptions (received in lower-case).
     * @example
     * normalizeName(' fernanDA  MONTENEGRO');
     * //=> "Fernanda Montenegro"
     *
     * normalizeName(' wilson da costa', ['da']);
     * //=> "Wilson da Costa"
     * @param {string} name - A `string` value.
     * @param {string} [exceptions] - A list of exceptions in lower-case.
     * @returns {string}
     */
    const normalizeName = (name, exceptions = []) => splitWords(name).map(capitalizeWordExcept(exceptions)).join(' ');

    /*!
     * @bitty/pipe v0.0.4
     * (c) Vitor Luiz Cavalcanti <vitorluizc@outlook.com> (https://vitorluizc.github.io/)
     * Released under the MIT License.
     */

    /**
     * Performs function composition in LTR (Left To Right) direction.
     * @example
     * const normalizeWhiteSpaces = text => name.replace(/\s+/g, ' ').trim();
     *
     * const getInitials = pipe(
     *   normalizeWhiteSpaces,
     *   name => name.split(' ').map(name => name.charAt(0)),
     *   initials => initials.join('').toLocaleUpperCase()
     * );
     *
     * getInitials('Vitor Luiz Cavalcanti');
     * //=> "VLC"
     * @param {Function} fn - An arity N function. Its result is the argument of next one.
     * @param {...Function[]} fns - Functions of arity 1. Each one's result is next's argument.
     * @returns {Function}
     */
    function pipe(fn) {
        var fns = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            fns[_i - 1] = arguments[_i];
        }
        return function () {
            return fns.reduce(function (x, fn) { return fn(x); }, fn.apply(null, arguments));
        };
    }

    /**
     * Get the last character of received text.
     * @param {string} text - A `string` value.
     * @returns {string}
     */
    const getLastChar = (text) => text.charAt(text.length - 1);
    /**
     * Add a period at end of a text, if you don't already have one.
     * @param {string} text - A `string` value.
     */
    const addPeriodAtEnd = (text) => getLastChar(text) === '.' ? text : text + '.';
    /**
     * Normalize a paragraph by normalizing its white-spaces, capitalizing first
     * letter and putting a period at end.
     * @example
     * normalizeParagraph(' once upon a time');
     * //=> "Once upon a time."
     * @param {string} value
     * @returns {string}
     */
    const normalizeParagraph = /*#__PURE__*/ pipe(normalizeWhiteSpaces, addPeriodAtEnd, capitalizeFirstLetter);

    /**
     * Resolve one or multiple texts into a single one.
     * @param {string | string[]} values - A `string` or an array of `string` values.
     * @returns {string}
     */
    const resolveToText = (values) => Array.isArray(values) ? values.join(' ') : values;
    /**
     * Transforms a text to lower case.
     * @param {string} text - A `string` value.
     * @returns {string}
     */
    const transformToLowerCase = (text) => text.toLocaleLowerCase();
    /**
     * Resolve received texts (when receives an `Array`) by normalizing its
     * white-spaces and its diacritics and transforming to lower-case.
     * @example
     * normalizeText(' so there\'s  a  Way to NORMALIZE ');
     * //=> "so there\'s a way to normalize"
     *
     * normalizeText(['Olá\r\n', 'como está a   senhorita?']);
     * //=> "ola como esta a senhorita?"
     * @param {string | string[]} values - A `string` or an array of `string` values.
     * @returns {string}
     */
    const normalizeText = /*#__PURE__*/ pipe(resolveToText, normalizeDiacritics, normalizeWhiteSpaces, transformToLowerCase);

    exports.capitalizeFirstLetter = capitalizeFirstLetter;
    exports.default = normalizeText;
    exports.normalizeDiacritics = normalizeDiacritics;
    exports.normalizeName = normalizeName;
    exports.normalizeParagraph = normalizeParagraph;
    exports.normalizeText = normalizeText;
    exports.normalizeWhiteSpaces = normalizeWhiteSpaces;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.umd.js.map
